#ifndef GRAPH_H
#define GRAPH_H
#include <map>
#include <boost/unordered_map.hpp>

namespace DataStructures{

	
	template <typename T_Node>
	class Graph
	{
	public:
		typedef unsigned int nodeId;
		typedef boost::unordered_map<unsigned int, T_Node > Nodes;
		typedef boost::unordered_multimap<unsigned int, unsigned int> Edges;
		
		Graph(unsigned int V, unsigned int E) :  _V(V), _E(E) {};

		//  
		// get the reference to a node
		//   if the node is not found it throws an exception
		// 

		T_Node & getNode(nodeId id) throw (int)
		{
			 Nodes::iterator it = _n.find(id); 
			 if (it != _n.end())
				 return it->second;
			 else
				 throw;
		};

		//
		// insert a node, 
		//   if is is already there the node is not inserted
		//
		void insertNode(const nodeId id, const T_Node t) 
		{ _n.insert(Nodes::value_type(id, t)); };

		// get an edge
		//  return iterators
		Edges::const_iterator getEdges(nodeId id) { return _e.find(id); }
		Edges::const_iterator edgesEnd() { return _e.end(); }

		// 
		// insert an edge, 
		//   if is is already there the node is not inserted
		//
		void insertEdge(const nodeId src, const nodeId dst)
		{ _e.insert(Edges::value_type(src, dst)); }

	private:
		unsigned int _V, _E; // number of nodes, number of edges
		Nodes _n;
		Edges _e;
	};


};

#endif